// %PUB_START%
// Licensed Materials - Property of IBM
// 
// 5724-H72, 5655-L82, 5724-L26
// 
// (c) Copyright IBM Corp. 2000, 2005
// 
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
// %PUB_END%
// @(#) jms/samples/jms/ws/sample3/Sample3Client.java, jms, j600, j600-200-060630 1.3.1.1 05/05/25 15:44:56

import java.util.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import sample3.Sample3;
import sample3.Sample3Home;

public class Sample3Client
{

  public static void main(String[] args)
  {
    String textMessage = "Default text message";
    String correlID;

    // check if a string was provided on the command line
    if (args.length > 0) {
      textMessage = args[0];
    }

    try {
      // Connect to the WebSphere service provider via JNDI
      Hashtable env = new Hashtable();
      env.put( Context.INITIAL_CONTEXT_FACTORY, "com.ibm.ejs.ns.jndi.CNInitialContextFactory" );
      env.put( Context.PROVIDER_URL, "iiop://localhost/" ); // Modify if server is non-local
      Context initial = new InitialContext(env);

      // Lookup the EJB home
      Object objref = initial.lookup("jmsSample3");
      Sample3Home home = 
        (Sample3Home)PortableRemoteObject.narrow(objref, Sample3Home.class);

      // Create the stub to talk to the EJB
      Sample3 testBean = home.create();

      // Need to have a durable subscription before we publish the message
      System.out.println("creating durable subscription");
      testBean.createSubscription();

      // Now we can publish
      System.out.println("calling putMessage");
      correlID = testBean.putMessage(textMessage);

      // And recive the message via a subscriber
      System.out.println("calling getMessage");
      String response = testBean.getMessage();

      System.out.println("got message '"+response+"'");

      // Remove the durable subscription
      System.out.println("deleting durable subscription");
      testBean.destroySubscription();

      System.out.println("done");

    } catch (Exception ex) {
      System.err.println("Caught an unexpected exception: "+ex);
      ex.printStackTrace();
    }
  }
}
