const static char sccsid[] = "@(#) lib/exits/oltp/pc/winnt/amqzscin.c, lqm, p600, p600-201-061220 1.20.1.1 05/05/26 23:28:34";
/******************************************************************************/
/*                                                                            */
/* Module Name: amqzscin.c                                                    */
/*                                                                            */
/* Description: WebSphere MQ XA switch program for CICS                       */
/*              XA Initialisation                                             */
/*                                                                            */
/* <N_OCO_COPYRIGHT>                                                          */
/* Licensed Materials - Property of IBM                                       */
/*                                                                            */
/* 63H9336                                                                    */
/* (c) Copyright IBM Corp. 1994, 2005 All Rights Reserved.                    */
/*                                                                            */
/* US Government Users Restricted Rights - Use, duplication or                */
/* disclosure restricted by GSA ADP Schedule Contract with                    */
/* IBM Corp.                                                                  */
/* <NOC_COPYRIGHT>                                                            */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/*                                                                            */
/* amqzscin.c is the XA Initialisation routine for CICS.                      */
/* It contains the following functions:                                       */
/*  CICS_XA_Init  -  CICS entry point for XA initialisation                   */
/*                                                                            */
/******************************************************************************/
/******************************************************************************/
/*                                                                            */
/* Note that the pre-build switch load file (Server: mqmc4swi.dll; Client:    */
/* mqcc4swi.dll) is built for CICS V4.3, to build a CICS V5.0 switch load     */
/* file, do the following:                                                    */
/*                                                                            */
/* For IBM WebSphere MQ V5.3 Server:                                          */
/*                                                                            */
/* create a file called amqzsc5.def with the following in:                    */
/*                                                                            */
/*    LIBRARY AMQZSC5                                                         */
/*    EXPORTS                                                                 */
/*    CICS_XA_Init                                                            */
/*                                                                            */
/* Then issue the following compiler statement if using Microsoft Visual C++  */
/* 6.0:                                                                       */
/*                                                                            */
/*    cl -c -Gz -GD amqzscin.c -Ic:\opt\encina\include                        */
/*                                                                            */
/* or the following compiler statement if running Microsoft Visual C++ .NET   */
/* 7.x:                                                                       */
/*                                                                            */
/*    cl -c -Gz amqzscin.c -Ic:\opt\encina\include                            */
/*                                                                            */
/* and then the following link statement:                                     */
/*                                                                            */
/*    link -nod -dll -out:amqzsc5.dll -def:amqzsc5.def amqzscin.obj \         */
/*         c:\opt\cics\lib\regxa_swxa.obj \                                   */
/*         c:\opt\cics\lib\libcicsrt.lib \                                    */
/*         c:\opt\encina\lib\libEncina.lib \                                  */
/*         c:\opt\encina\lib\libEncServer.lib \                               */
/*         "c:\Program Files\dce\dcelocal\lib\libdce.lib" \                   */
/*         "c:\Program Files\dce\dcelocal\lib\pthreads.lib" \                 */
/*         "c:\Program Files\IBM\WebSphere MQ\Tools\Lib\mqmcics4.lib" \       */
/*         msvcrt.lib kernel32.lib                                            */
/*                                                                            */
/* For IBM WebSphere MQ Extended Transactional Client:                        */
/*                                                                            */
/* create a file called amqczsc5.def with the following in:                   */
/*                                                                            */
/*    LIBRARY AMQCZSC5                                                        */
/*    EXPORTS                                                                 */
/*    CICS_XA_Init                                                            */
/*                                                                            */
/* Then issue the following compiler statement if using Microsoft Visual C++  */
/* 6.0:                                                                       */
/*                                                                            */
/*    cl -c -Gz -GD amqzscin.c -Ic:\opt\encina\include                        */
/*                                                                            */
/* or the following compiler statement if running Microsoft Visual C++ .NET   */
/* 7.x:                                                                       */
/*                                                                            */
/*    cl -c -Gz amqzscin.c -Ic:\opt\encina\include                            */
/*                                                                            */
/* and then the following link statement:                                     */
/*                                                                            */
/*    link -nod -dll -out:amqczsc5.dll -def:amqczsc5.def amqzscin.obj \       */
/*         c:\opt\cics\lib\regxa_swxa.obj \                                   */
/*         c:\opt\cics\lib\libcicsrt.lib \                                    */
/*         c:\opt\encina\lib\libEncina.lib \                                  */
/*         c:\opt\encina\lib\libEncServer.lib \                               */
/*         "c:\Program Files\dce\dcelocal\lib\libdce.lib" \                   */
/*         "c:\Program Files\dce\dcelocal\lib\pthreads.lib" \                 */
/*         "c:\Program Files\IBM\WebSphere MQ\Tools\Lib\mqccics4.lib" \       */
/*         msvcrt.lib kernel32.lib                                            */
/*                                                                            */
/******************************************************************************/


/******************************************************************************/
/* Includes                                                                   */
/******************************************************************************/

/******************************************************************************/
/* Select compiler specific header files                                      */
/******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <cmqc.h>

/******************************************************************************/
/* Select XA header file                                                      */
/******************************************************************************/

#include <tmxa/xa.h>

/******************************************************************************/
/* External Data declarations                                                 */
/******************************************************************************/
#define CALLCONV __cdecl
#define DLLIMPORT __declspec(dllimport)
extern DLLIMPORT struct xa_switch_t MQRMIXASwitch;
extern DLLIMPORT struct xa_switch_t MQRMIXASwitchDynamic;
extern struct xa_switch_t RegXA_xa_switch;
extern struct xa_switch_t *cics_xa_switch;

/******************************************************************************/
/* Data defined in this module                                                */
/******************************************************************************/

/******************************************************************************/
/* Local function declarations                                                */
/******************************************************************************/

extern void MQENTRY AMQCICSINIT(void *);
extern void CALLCONV cics_xa_init(void);

/******************************************************************************/
/* Function Definitions                                                       */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* Function Name:  amqzscin                                                   */
/*                                                                            */
/* Description: CICS XA initialisation.                                       */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/*                                                                            */
/*  amqzscin is called by CICS to initialise XA support for WebSphere MQ      */
/*                                                                            */
/* Input Parameters:  None                                                    */
/*                                                                            */
/* Output Parameters: None                                                    */
/*                                                                            */
/* InOut Parameters:  None                                                    */
/*                                                                            */
/* Returns:           RegXA_xa_switch                                         */
/*                                                                            */
/******************************************************************************/
struct xa_switch_t * CALLCONV CICS_XA_Init(void)
{
  /****************************************************************************/
  /* Set CICS pointer to our structure                                        */
  /****************************************************************************/
  cics_xa_switch = &MQRMIXASwitchDynamic;

  /****************************************************************************/
  /* Initialise CICS support                                                  */
  /****************************************************************************/
  cics_xa_init();

  /****************************************************************************/
  /* Tell MQ that CICS is available.                                          */
  /* This call will cause MQ to callback to CICS on any MQCONN to determine   */
  /* the CICS UserId and transaction name, so that it can be used in          */
  /* authority checking and context.                                          */
  /* For compatability with OS/2, a dummy parameter is given.                 */
  /****************************************************************************/
  AMQCICSINIT(NULL);

  /****************************************************************************/
  /* Return from function                                                     */
  /****************************************************************************/
  return(&RegXA_xa_switch);

}

/******************************************************************************/
/*                                                                            */
/* End of amqzscin.c                                                          */
/*                                                                            */
/******************************************************************************/
