#
# Name: XASWIT.MAK
#
# Description: Windows NT makefile for DB2, Oracle, Informix and Sybase XA switch load files
#
# <N_OCO_COPYRIGHT>
#
# Licensed Materials - Property of IBM
#
# 04L1830, 5639-B43
# (C) Copyright IBM Corp. 1996, 2004
#
# The source code for this program is not published or other-
# wise divested of its trade secrets, irrespective of what has
# been deposited with the U.S. CopyRight Office
#
# <NOC_COPYRIGHT>
#
# To make the DB2 XA switch load file, run the command:-
#                nmake -f xaswit.mak db2swit.dll
#
# To make the Sybase XA switch load file, run the command:-
#                nmake -f xaswit.mak sybswit.dll
#
# To make the Oracle XA switch load file, run the command:-
#                nmake -f xaswit.mak oraswit.dll
# To make the Oracle XA Dynamic Registration switch load file, run the command:-
#                nmake -f xaswit.mak oraswitd.dll
#
# To make the Informix XA switch load file, run the command:-
#                nmake -f xaswit.mak infswit.dll
#
# Note: If your database library has a different name, or is on a different
#       drive, or is in a different directory to the one listed in the xxxLIBPATH
#       statement, you will need to alter that statement.
#
!include <ntwin32.mak>

#-------------------------------------------------------------------------------
# The switch load file must be built as 32-bit, PROCESSOR_ARCHITECTURE being
# set to AMD64 indicates that a 64-bit compiler is in use.
#-------------------------------------------------------------------------------
!if ("$(PROCESSOR_ARCHITECTURE)" == "AMD64")
!error Switch load files used by WebSphere MQ must be 32-bit
!endif

#-------------------------------------------------------------------------------
# If the MQEXITDIR has not been overridden, work out the default exit directory
#
# The default product install location differs depending on whether the
# operating system is 32-bit or 64-bit. We can determine this because on 64-bit
# operating systems, either PROCESSOR_ARCHITEW6432 is defined, or
# PROCESSOR_ARCHITECTURE is set to AMD64.
#
# Note: If you have installed WebSphere MQ into a different location to the
#       default, or otherwise want to override the location to which the exit is
#       built, you can do so by setting the MQEXITDIR macro, either on the
#       command line, or before this point in the make file.
#-------------------------------------------------------------------------------
!if !defined(MQEXITDIR)

!if defined(PROCESSOR_ARCHITEW6432)

MQEXITDIR="C:\Program Files (x86)\IBM\WebSphere MQ\exits"

!else

MQEXITDIR="C:\Program Files\IBM\WebSphere MQ\exits"

!endif

!endif

#-------------------------------------------------------------------------------
# DB2 XA switch load file
#-------------------------------------------------------------------------------
DB2LIBS=db2api.lib

DB2LIBPATH="C:\Program Files\IBM\SQLLIB\lib"

DB2LIBFLAGS=$(DB2LIBPATH)\$(DB2LIBS)

db2swit.lib db2swit.exp: $*.obj $*.def
                         $(implib) -machine:$(CPU) \
                         -def:$*.def $*.obj

$(MQEXITDIR)\db2swit.dll: $*.obj $*.def $*.exp
                          $(link) $(dlllflags)                                \
                          -base:0x1C000000                                    \
                          $*.exp $*.obj                                       \
                          -out:$(MQEXITDIR)\db2swit.dll                       \
                          $(conlibsdll) $(DB2LIBFLAGS)
                          if exist $(MQEXITDIR)\db2swit.dll.manifest          \
                          mt.exe  -manifest $(MQEXITDIR)\db2swit.dll.manifest \
                          -outputresource:$(MQEXITDIR)\db2swit.dll;#2

#-------------------------------------------------------------------------------
# Sybase XA switch file
#-------------------------------------------------------------------------------
SYBLIBS=libxadtm.lib

SYBLIBPATH=C:\sybase\OCS-12_5\lib

SYBLIBFLAGS=$(SYBLIBPATH)\$(SYBLIBS)

sybswit.lib sybswit.exp: $*.obj $*.def
                         $(implib) -machine:$(CPU) \
                         -def:$*.def $*.obj

$(MQEXITDIR)\sybswit.dll: $*.obj $*.def $*.exp
                          $(link) $(dlllflags)                                \
                          -base:0x1C000000                                    \
                          $*.exp $*.obj                                       \
                          -out:$(MQEXITDIR)\sybswit.dll                       \
                          $(conlibsdll) $(SYBLIBFLAGS)
                          if exist $(MQEXITDIR)\sybswit.dll.manifest          \
                          mt.exe  -manifest $(MQEXITDIR)\sybswit.dll.manifest \
                          -outputresource:$(MQEXITDIR)\sybswit.dll;#2

#-------------------------------------------------------------------------------
# Oracle XA switch load file
#-------------------------------------------------------------------------------
ORALIBS=ORAXA9.LIB
# Comment out the definition of ORALIBS above and uncomment the defenition below
# if using Oracle 10g
#ORALIBS=ORAXA10.LIB

# If the ORACLE_HOME environment is set, we will use it, otherwise use a hard coded path
!ifdef ORACLE_HOME

ORALIBPATH=$(ORACLE_HOME)\rdbms\XA

!else

ORALIBPATH=C:\oracle\ora92\rdbms\XA

# Comment out the definition of ORALIBPATH above and uncomment the definition below
# if using Oracle 10gR1. Note the first default database instance Oracle creates has
# been used (db_1), this may need changing if this instance name does not exist in
# the Oracle 10gR1 sub directory (10.1.0).
#ORALIBPATH=C:\oracle\product\10.1.0\db_1\rdbms\XA

# Comment out the definition of ORALIBPATH above and uncomment the definition below
# if using Oracle 10gR2. Note the first default database instance Oracle creates has
# been used (db_1), this may need changing if this instance name does not exist in
# the Oracle 10gR2 sub directory (10.2.0).
#ORALIBPATH=C:\oracle\product\10.2.0\db_1\rdbms\XA

!endif

ORALIBFLAGS=$(ORALIBPATH)\$(ORALIBS)

oraswit.lib oraswit.exp: $*.obj $*.def
                         $(implib) -machine:$(CPU) \
                         -def:$*.def $*.obj

$(MQEXITDIR)\oraswit.dll: $*.obj $*.def $*.exp
                          $(link) $(dlllflags)                               \
                          -base:0x1C000000                                   \
                          $*.exp $*.obj                                      \
                          -out:$(MQEXITDIR)\oraswit.dll                      \
                          $(conlibsdll) $(ORALIBFLAGS)
                          if exist $(MQEXITDIR)\oraswit.dll.manifest          \
                          mt.exe  -manifest $(MQEXITDIR)\oraswit.dll.manifest \
                          -outputresource:$(MQEXITDIR)\oraswit.dll;#2

#-------------------------------------------------------------------------------
# Oracle XA Dynamic Registration switch load file
#-------------------------------------------------------------------------------

oraswitd.lib oraswitd.exp: $*.obj $*.def
                           $(implib) -machine:$(CPU) \
                           -def:$*.def $*.obj

$(MQEXITDIR)\oraswitd.dll: $*.obj $*.def $*.exp
                           $(link) $(dlllflags)                                 \
                           -base:0x1C000000                                     \
                           $*.exp $*.obj                                        \
                           -out:$(MQEXITDIR)\oraswitd.dll                       \
                           $(conlibsdll) $(ORALIBFLAGS)
                           if exist $(MQEXITDIR)\oraswitd.dll.manifest          \
                           mt.exe  -manifest $(MQEXITDIR)\oraswitd.dll.manifest \
                           -outputresource:$(MQEXITDIR)\oraswitd.dll;#2

#-------------------------------------------------------------------------------
# Informix XA switch load file
#-------------------------------------------------------------------------------
INFLIBS=isqlt09a.lib

INFLIBPATH="C:\Program Files\Informix\Client-SDK\lib"

INFLIBFLAGS=$(INFLIBPATH)\$(INFLIBS)

infswit.lib infswit.exp: $*.obj $*.def
                         $(implib) -machine:$(CPU) \
                         -def:$*.def $*.obj

$(MQEXITDIR)\infswit.dll: $*.obj $*.def $*.exp
                          $(link) $(dlllflags)                                \
                          -base:0x1C000000                                    \
                          $*.exp $*.obj                                       \
                          -out:$(MQEXITDIR)\infswit.dll                       \
                          $(conlibsdll) $(INFLIBFLAGS)
                          if exist $(MQEXITDIR)\infswit.dll.manifest          \
                          mt.exe  -manifest $(MQEXITDIR)\infswit.dll.manifest \
                          -outputresource:$(MQEXITDIR)\infswit.dll;#2

#-------------------------------------------------------------------------------

.c.obj:
    $(cc) $(cflags) $(cvarsdll) $*.c
