const static char sccsid[] = "@(#) samples/dotnet/mcp/mmqsget.cpp, dotnet, p600, p600-201-061220 1.6.1.2 06/04/28 10:57:56";
// Library:       WebSphere MQ
// Component:     MQ.NET Sample Programs
// Part:          MMQSGET.CPP
//
// Description:   Sample Managed C++ program that gets messages from a named
//                queue.
//
//                MMQSGET has 3 parameters:
//                - the name of a queue (required)
//                - the name of a queue manager (optional)
//                - the definition of a channel (optional)
//
//                If no queue manager name is given, the queue manager
//                defaults to the default local queue manager. If a
//                channel is defined, it should have the same format
//                as the MQSERVER environment variable.
//  <N_OCO_COPYRIGHT> 
//  Licensed Materials - Property of IBM
//  
//  63H9336
//  (c) Copyright IBM Corp. 1994, 2005 All Rights Reserved.
//  
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with
//  IBM Corp.
//  <NOC_COPYRIGHT> 

#using <mscorlib.dll>
#using <amqmdnet.dll>

#include <tchar.h>

using namespace System;
using namespace IBM::WMQ;

// This is the entry point for this application
int _tmain(int argc, char * * argv )
{
	MQQueueManager*      mqQMgr;         // MQQueueManager instance
	MQQueue*             mqQueue;        // MQQueue instance
	String*              queueName;      // Name of queue to use

	System::Console::WriteLine( S"Sample MMQSGET start" );
	if (argc < 2) 
	{
		System::Console::WriteLine( S"Required parameter missing - queue name" );
		return( (int)99 );
	}
	else 
	{
		queueName = argv[1];
	}

	///
	/// Try to create an MQQueueManager instance 
	/// 
	try 
	{
		if (argc > 3) 
		{
			// queue name, queue manager name, channel definition all provided

			// Break down the channel definition,
			// which is of the form "channel-name/transport-type/connection-name".
			String* channelDefinition = argv[3];
		    String* channelName = "";
			String* transportType = "";
			String* connectionName = "";
			Char separator[] = {'/'};
			String* parts[];
			parts = channelDefinition->Split( separator );
			if (parts->Length > 0) channelName = parts[0];
			if (parts->Length > 1) transportType = parts[1];
			if (parts->Length > 2) connectionName = parts[2];
			mqQMgr = new MQQueueManager( argv[2], channelName, connectionName );
		}
		else
			if (argc == 3) 
		{
			// queue name, queue manager name provided
			mqQMgr = new MQQueueManager( argv[2] );
		}
		else
		{
			// queue name provided - use default queue manager
			mqQMgr = new MQQueueManager();
		}
	}
	catch (MQException* mqe) 
	{
		// stop if failed
		System::Console::WriteLine( S"create of MQQueueManager ended with {0}", mqe->Message );
		return( (int)mqe->Reason );
	}

	///
	/// Try to open the queue
	///
	try 
	{
		mqQueue = mqQMgr->AccessQueue( queueName, 
			                           MQC::MQOO_INPUT_AS_Q_DEF          // open queue for input
			                           + MQC::MQOO_FAIL_IF_QUIESCING );  // but not if MQM stopping
	}
	catch (MQException* mqe) 
	{
		// stop if failed
		System::Console::WriteLine( S"MQQueueManager::AccessQueue ended with {0}", mqe->Message );
		return( (int)mqe->Reason );
	}


	///
	/// Get messages from the message queue
	/// Loop until there is a failure
	/// 
	bool isContinue = true;
	while (isContinue) 
	{
		MQMessage*           mqMsg;          // MQMessage instance
		MQGetMessageOptions* mqGetMsgOpts;   // MQGetMessageOptions instance

		mqMsg = new MQMessage();
		mqGetMsgOpts = new MQGetMessageOptions();
		mqGetMsgOpts->WaitInterval = 15000;  // 15 second limit for waiting
		mqGetMsgOpts->Options |= MQC::MQGMO_WAIT;
		try 
		{
			mqQueue->Get( mqMsg, mqGetMsgOpts );
			if (String::Compare(mqMsg->Format, MQC::MQFMT_STRING->Substring(0, mqMsg->Format->Length)) == 0) 
			{
				System::Console::WriteLine(  mqMsg->ReadString(mqMsg->MessageLength) );
			}
			else 
			{
				System::Console::WriteLine( S"Non-text message" );
			}
		}
		catch (MQException* mqe) 
		{
			// report reason, if any
			if ( mqe->Reason == MQC::MQRC_NO_MSG_AVAILABLE ) 
			{
				// special report for normal end
				System::Console::WriteLine( S"no more messages" );
				isContinue = false;
			} 
			else 
			{
				// general report for other reasons
				System::Console::WriteLine( S"MQQueue::Get ended with {0}", mqe->Message );

				// treat truncated message as a failure for this sample
				if ( mqe->Reason == MQC::MQRC_TRUNCATED_MSG_FAILED ) 
				{
					isContinue = false;
				}
			}
		}
	}
	try
	{
		//Close the Queue
		mqQueue->Close();

		//Disconnect the Queue Manager
		mqQMgr->Disconnect();
	}
	catch (MQException* mqe) 
	{
		System::Console::WriteLine( S"A WebSphere MQ error occurred with {0}", mqe->Message );
		return( (int)mqe->Reason );
	}

	System::Console::WriteLine( S"Sample MMQSGET end" );
	return (0);
}
