Attribute VB_Name = "SampleUtils"
'<!/******************************************************************************/!>
'<!/*                                                                            */!>
'<!/* Program name: VbMqMts                                                      */!>
'<!/*                                                                            */!>
'<!/* Description:  Sample VB program for WebSphere MQ MTS Support               */!>
'<!/* <N_OCO_COPYRIGHT>                                                          */!>
'<!/* Licensed Materials - Property of IBM                                       */!>
'<!/*                                                                            */!>
'<!/* 5724-B41                                                                   */!>
'<!/* (C) Copyright IBM Corp. 1999, 2002 All Rights Reserved.                    */!>
'<!/*                                                                            */!>
'<!/* US Government Users Restricted Rights - Use, duplication or                */!>
'<!/* disclosure restricted by GSA ADP Schedule Contract with                    */!>
'<!/* IBM Corp.                                                                  */!>
'<!/* <NOC_COPYRIGHT>                                                            */!>
'<!/******************************************************************************/!>
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' This contains all sorts of bits to make it work (or more refined)
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''


Option Explicit

' Various useful colours
Global Const COLA = &HDD3CDD
Global Const GREY128 = &H808080
Global Const MUTEEDGE = &HA0A0A0
Global Const MUTELINE = &HB0B0B0
Global Const GREY192 = &HC0C0C0
Global Const GREY208 = &HD0D0D0
Global Const WHITE = &HFFFFFF
Global Const BLACK = 0

' Option to control what kind of highlighting.
Public Enum ToHighlightEnum
    HNone = 0
    HRequestTransfer
    HBackEnd
    HFind
    HStatus
    HAll
End Enum

Dim CurrentHighlight As ToHighlightEnum




Public Sub SimulateBackEnd(ServerLog As ListBox)
    ' Simulate execution of the back end server.
    ' This code is here to save needing another executable,
    ' and to keep the demo neat.
    
    Dim mqs As New MQSession
    Dim qm As MQQueueManager
    Dim inq As MQQueue
    Dim outq As MQQueue
    Dim inmsg As MQMessage
    Dim outmsg As MQMessage
    Dim pmo As MQPutMessageOptions
    Dim gmo As MQGetMessageOptions
    Dim trancode As String
    Dim status As String
    
    
    On Error GoTo eee
    
    ' set up MQ objects for input
    Set qm = mqs.AccessQueueManager("")
    Set inq = qm.AccessQueue("BankRequest", MQOO_INQUIRE Or MQOO_INPUT_AS_Q_DEF)
    
    Do  ' once we are here, process all available messages
        ' set up and get the input message
        Set inmsg = mqs.AccessMessage
        Set outmsg = mqs.AccessMessage
        Set gmo = mqs.AccessGetMessageOptions
        
        gmo.Options = MQGMO_SYNCPOINT
        inq.Get inmsg, gmo
        If inq.ReasonCode = MQRC_NO_MSG_AVAILABLE Then Exit Do
        
        ' process input message
        trancode = Left(inmsg.MessageData, 4)
        If trancode = "ATR1" Then
            If Rnd < 0.9 Then
                status = "done OK"
            Else
                status = "insufficient funds"
            End If
        Else
            ' in real world, send administartive errors!
            status = "bad request"
        End If
        outmsg.MessageData = inmsg.MessageData & status
        
        ' prepare output
        Set pmo = mqs.AccessPutMessageOptions
        pmo.Options = MQPMO_SYNCPOINT
        
        Set outq = qm.AccessQueue( _
            inmsg.ReplyToQueueName, _
            MQOO_INQUIRE Or MQOO_OUTPUT, _
            inmsg.ReplyToQueueManagerName)
        
        ' should be more stuff in here to test relationship of ids,
        ' and to copy priority, timeout etc.
        ' Not included in this simple sample
        outmsg.CorrelationId = inmsg.MessageId
        outmsg.Persistence = inmsg.Persistence
        
        outq.Put outmsg, pmo
        ServerLog.AddItem outmsg.MessageData
    Loop
    
    ' commit all messages as single batch for efficiency
    ' (could commit inside loop)
    qm.Commit
    ServerLog.AddItem "------------------------"
    
    RefreshDisplay
    
    ' could clean up here
    Exit Sub
    
eee:
    ' handle all errors, but no messages is not an error
    If mqs.ReasonCode = MQRC_NO_MSG_AVAILABLE Then Resume Next
    
    Beep
    VbMqMtsClientForm.BackEndFrame = "Server error: " & Err.Description
    ServerLog.AddItem VbMqMtsClientForm.BackEndFrame
    Exit Sub
End Sub

' Check for errors, and return a string reporting any error
Public Function CheckErr() As String
    Dim e As String
    If Err.Number = 0 Then Exit Function
    e = "Error " & Err.Number & " x" & Hex(Err.Number) & " " & Err.Description
    log e
    CheckErr = e
    Err.Clear
End Function


' Set an item in a list box.
' Do not disturb if already correct.
' Clear rest of list if different.
' Indicate whether a change was made.
Public Function setitem(l As ListBox, k As Integer, str As String) As Boolean
    Dim i As Integer
    
    ' if unchanged just quit
    If i < l.ListCount And l.List(k) = str Then Exit Function
    
    ' clear the rest (if any) and set the item
    setitem = True
    For i = l.ListCount - 1 To k Step -1
        l.RemoveItem i
    Next i
    l.AddItem str
End Function

' Draw a pretty line. end points are relative to objects
' If col=0 then draw all in grey
Public Sub DrawLine(obj1, x1 As Single, y1 As Single, obj2, x2 As Single, y2 As Single, ByVal col As Long)
    Dim xx1 As Long, yy1 As Long ' start coords
    Dim xx2 As Long, yy2 As Long ' end coords
    Dim xxm As Long, yym As Long ' kink coords
    Dim dx As Long, dy As Long    ' direction -1 or 1
    Dim i As Integer, j As Integer ' indices for arrow
    Dim col2 As Long    ' colour for edges
    
    ' check colour style
    If col = MUTELINE Then
        col2 = MUTEEDGE
    Else
        col2 = BLACK
    End If
    
    ' shorthand object to draw onto
    Dim f
    Set f = VbMqMtsClientForm
    f.DrawWidth = 1
    
    ' compute 'real' endpoints
    xx1 = obj1.Left + x1 * obj1.Width
    yy1 = obj1.Top + y1 * obj1.Height
    xx2 = obj2.Left + x2 * obj2.Width
    yy2 = obj2.Top + y2 * obj2.Height
    If x1 = -999 Then xx1 = xx2
    If y1 = -999 Then yy1 = yy2
    If x2 = -999 Then xx2 = xx1
    If y2 = -999 Then yy2 = yy1
    
    ' two basic cases, vertical or horizontal/diag
    ' use several single width lines to force clean ends
    ' similarly draw arrowheads with pixels to force clean heads
    If xx1 = xx2 Then
        If yy1 < yy2 Then dy = 1 Else dy = -1
        f.Line (xx1 + 2, yy1)-(xx2 + 2, yy2 - 4 * dy), col2
        f.Line (xx1 - 2, yy1)-(xx2 - 2, yy2 - 4 * dy), col2
        
        f.Line (xx1 + 1, yy1)-(xx2 + 1, yy2 - 4 * dy), col
        f.Line (xx1 + 0, yy1)-(xx2 + 0, yy2 - 4 * dy), col
        f.Line (xx1 - 1, yy1)-(xx2 - 1, yy2 - 4 * dy), col
        
        For i = 0 To 4
            For j = 0 To i * 2
                f.PSet (xx2 - i + j, yy2 - i * dy), col2
            Next j
        Next i
        
    Else ' horizontal/diag
        
        xxm = (xx1 + xx2) / 2
        
        If xx1 < xx2 Then dx = 1 Else dx = -1
        If yy1 < yy2 Then dy = 1 Else dy = -1
        
        f.Line (xx1, yy1 + 2)-(xxm + 2 * dx, yy1 + 2), col2
        f.Line (xx1, yy1 - 2)-(xxm + 2 * dx, yy1 - 2), col2
        f.Line (xxm + 2, yy1 - 2 * dy)-(xxm + 2, yy2 + 2 * dy), col2
        f.Line (xxm - 2, yy1 - 2 * dy)-(xxm - 2, yy2 + 2 * dy), col2
        f.Line (xxm - 2 * dx, yy2 + 2)-(xx2 - 4 * dx, yy2 + 2), col2
        f.Line (xxm - 2 * dx, yy2 - 2)-(xx2 - 4 * dx, yy2 - 2), col2
        
        f.Line (xx1, yy1 + 1)-(xxm + 1 * dx, yy1 + 1), col
        f.Line (xx1, yy1 + 0)-(xxm + 1 * dx, yy1 + 0), col
        f.Line (xx1, yy1 - 1)-(xxm + 1 * dx, yy1 - 1), col
        f.Line (xxm + 1, yy1 - 1 * dy)-(xxm + 1, yy2 + 1 * dy), col
        f.Line (xxm + 0, yy1 - 1 * dy)-(xxm + 0, yy2 + 1 * dy), col
        f.Line (xxm - 1, yy1 - 1 * dy)-(xxm - 1, yy2 + 1 * dy), col
        f.Line (xxm - 1 * dx, yy2 + 1)-(xx2 - 4 * dx, yy2 + 1), col
        f.Line (xxm - 1 * dx, yy2 + 0)-(xx2 - 4 * dx, yy2 + 0), col
        f.Line (xxm - 1 * dx, yy2 - 1)-(xx2 - 4 * dx, yy2 - 1), col
        
        f.DrawWidth = 1
        ' f.ForeColor = col2
        For i = 0 To 4
            For j = 0 To i * 2
                f.PSet (xx2 - i * dx, yy2 - i + j), col2
            Next j
            'f.Line Step(1 - i, 1)-Step(i, 0), 0
        Next i
    End If ' horizontal/diag
End Sub


' Create given local queue on the given queue manger
Public Function CreateQueue(qname As String, QMName As String) As Boolean
    Dim mqs As New MQSession
    Dim qm  As MQAX200.MQQueueManager
    
    Dim bag As New MQAIX100.MQBag   ' bag to ask query
    Dim resBag As MQAIX100.MQBag    ' result bag of bags
    Dim res As MQAIX100.MQBag       ' result bag
    Dim rc As Long                  ' return code
    
    CreateQueue = True
    On Error GoTo eee
    
    ' Connect to queue manager
    Set qm = mqs.AccessQueueManager(QMName)
    
    ' Set up Create Queue bag
    bag.Options = MQCBO_ADMIN_BAG
    bag.Add MQQT_LOCAL, MQIA_Q_TYPE
    bag.Add qname, MQCA_Q_NAME
    
    ' Execute the CreateQueue
    Set resBag = bag.Execute(qm, MQCMD_CREATE_Q)
    
    ' Check return code.
    Set res = resBag.Item(, 0)
    rc = res.Item(MQIASY_REASON)
    If rc = MQRCCF_OBJECT_ALREADY_EXISTS Then
        log "Queue '" & qname & "' already exists."
    ElseIf rc Then
        log "FAILED Create queue " & qname
        log " ... " & rc & " " & mqs.ReasonCodeName(rc)
    Else
        log "Created queue " & qname
    End If
    Exit Function
    
eee:
    log "FAILED to create queue " & qname
    log " ... " & Err.Description
    If Err.Description = "MQRC_CMD_SERVER_NOT_AVAILABLE" Then
        MsgBox "The command server for this queue manager is not available.  Please start the command server before attempting to setup this demo again.", vbOKOnly + vbCritical
    Else
        MsgBox "Could not create a queue on this queue manager, reason " + Err.Description, vbCritical + vbOKOnly
    End If
    
    CreateQueue = False
   
    Exit Function
End Function

' Create standard BankRequest table in given database
Public Function CreateTable(tabname As String, connstr As String) As Boolean

    Dim Cn As ADODB.Connection
    
    CreateTable = True
    
    If connstr = "" Then
        log "Database table not set up, no database specified"
        Exit Function
    End If
    
    On Error GoTo eee

    Set Cn = New ADODB.Connection
    Cn.Open connstr
    
    Cn.Execute "create table " & tabname & " (ID char(48), FromAccount char(8), ToAccount char(8), Amount integer, Status char(20), ChangeTime char(20) )"
   
    Cn.Close
    
    Set Cn = Nothing
    log "Database table " & tabname & " set up for connection " & connstr
    Exit Function
    
eee:
    Dim e As String
    If Err = &H80040E14 Then
        log "Table " & tabname & " already exists for connection " & connstr
    Else
        log "FAILED to create database table " & tabname & " set up for connection " & connstr
        log " ... " & Err.Number & " " & Err.Description
    End If
    Set Cn = Nothing
    CreateTable = False
    
    Exit Function
End Function

' refresh values from DB relation BankRequest
Public Sub DoDBShow(DBShow As ListBox, ConnectionString As String)
    Dim rr As String ' for errors
    Static Cn As ADODB.Connection
    Static oConnectionString As String
    Dim rs As New ADODB.Recordset
    Dim f As ADODB.Field
    Dim s As String ' for output string
    Dim i As Integer  ' count as we move down

    On Error GoTo eee

    ' cope with no database case
    If ConnectionString = "" Then
        setitem DBShow, 0, "No DB specified."
        Exit Sub
    End If
    
    ' cope with changed database case, make sure DB open
    If oConnectionString <> ConnectionString Then
        oConnectionString = ConnectionString
        
        If Cn Is Nothing Then
            Set Cn = New ADODB.Connection
        Else
            Cn.Close
        End If
        
        Cn.Open VbMqMtsClientForm.DBConnection.Text
        rr = rr & CheckErr
    End If
   
    
    ' now read from the database
    rs.Open "SELECT FromAccount, ToAccount, Amount, Status FROM BankRequest", Cn ' , adOpenDynamic, adLockPessimistic
    rr = rr & CheckErr
    
    setitem DBShow, 0, "FromAcc. .ToAcc.. Amount .Status.."
    On Error Resume Next
        rs.MoveFirst
        ' If Err.Number = 3021 Then setitem DBShow, 1, "---- no records in table": i = 1
    On Error GoTo eee
    For i = 1 To 100
        If rs.EOF Then Exit For
        Dim sep As String
        sep = " "
        s = ""
        s = s & Left(rs!FromAccount, 8) & sep
        s = s & Left(rs!ToAccount, 8) & sep
        s = s & Format(rs!amount, "###0.00") & sep
        s = s & Left(rs!status, 9) & sep
        'For Each f In rs.Fields
        '    s = s & f.Value & " | "
        'Next f
        setitem DBShow, i, s
        rs.MoveNext
    Next i
    setitem DBShow, i, "--------+--------+------+---------"
    'Cn.Close
    'rr = rr & checkerr
    
    Set rs = Nothing
    ' Set Cn = Nothing
    Exit Sub
    
eee:
    setitem DBShow, i, "!!!!!!!!!!"
    setitem DBShow, i + 1, "Error " & Err.Number & " x" & Hex(Err.Number)
    setitem DBShow, i + 2, Err.Description
    Set rs = Nothing
    Set Cn = Nothing
    Exit Sub

End Sub

' Clear all records (but not table) from database for demo reset.
Public Sub DoDBClear(DBShow As ListBox)

    Dim rr As String ' for errors
    Static Cn As ADODB.Connection

    If VbMqMtsClientForm.DBConnection.Text = "" Then
        setitem DBShow, 0, "No DB specified."
        Exit Sub
    End If
    
    'If Cn Is Nothing Then
        Set Cn = New ADODB.Connection
        Cn.Open VbMqMtsClientForm.DBConnection.Text
        rr = rr & CheckErr
    'End If
        
    Cn.Execute "Delete From BankRequest;"
    
    Cn.Close
    rr = rr & CheckErr
    
    Set Cn = Nothing
    Exit Sub
    
eee:
    setitem DBShow, 0, "!!! empty DB error !!!"
    setitem DBShow, 1, "Error " & Err.Number & " x" & Hex(Err.Number)
    setitem DBShow, 2, Err.Description
    Set Cn = Nothing
    Exit Sub

End Sub

' Refresh the display of the queues and database
Public Sub RefreshDisplay()
    With VbMqMtsClientForm
        If .ShowBankRequest.Refresh Then Beep
        If .ShowBankReply.Refresh Then Beep
        DoDBShow .DBShow, .DBConnection
    End With
End Sub

' Make a graphics object look like a window, with given caption.
Public Sub DrawWindow(f, caption As String)
Dim w As Long, h As Long
    w = f.Width
    h = f.Height
    f.BorderStyle = 0
    f.BackColor = GREY208
    f.Line (0, 0)-(w, 0), GREY208
    f.Line (0, 0)-(0, h), GREY208
    f.Line (1, 1)-(w, 1), WHITE
    f.Line (1, 1)-(1, h), WHITE
    
    f.Line (w - 1, 0)-(w - 1, h), BLACK
    f.Line (0, h - 1)-(w, h - 1), BLACK
    f.Line (w - 2, 1)-(w - 2, h), GREY128
    f.Line (1, h - 2)-(w, h - 2), GREY128
    
    f.Line (4, 4)-(w - 5, 19), COLA, BF

    f.ForeColor = WHITE
    f.CurrentX = 8
    f.CurrentY = 5
    f.Print caption

End Sub

' Draw connection lines onto the form.
Public Sub DrawLines(hi As ToHighlightEnum)
Dim col As Long

    If hi <> CurrentHighlight Then
    
        With VbMqMtsClientForm
        
            ' First highlight MTS objects as appropriate,
            ' The refresh forces display soon,
            ' otherwise lags behind lines drawn on Form.
            If hi = HRequestTransfer Or hi = HAll Then
                .MTSRq.BackColor = COLA
            Else
                .MTSRq.BackColor = WHITE
            End If
            .MTSRq.Refresh
        
            If hi = HStatus Or hi = HAll Then
                .MTSQs.BackColor = .MTSFrame.BackColor
            Else
                .MTSQs.BackColor = WHITE
            End If
            .MTSQs.Refresh
        
            If hi = HFind Or hi = HStatus Or hi = HAll Then
                .MTSFr.BackColor = .MQFrame.BackColor
            Else
                .MTSFr.BackColor = WHITE
            End If
            .MTSFr.Refresh
    
            ' Draw dividing lines
            Dim x1 As Long, x2 As Long, y As Long
            x1 = (.StatusFrame.Left + .StatusFrame.Width + .ShowBankReply.Left) / 2
            x2 = (.TQOut.Left + .TQOut.Width + .BackEndFrame.Left) / 2
            y = (.StatusFrame.Top + .StatusFrame.Height + .DBConnection.Top) / 2
            
            VbMqMtsClientForm.Line (x1, 0)-(x1, y), BLACK
            VbMqMtsClientForm.Line (x1 + 1, 0)-(x1 + 1, y), WHITE
            VbMqMtsClientForm.Line (x2, 0)-(x2, y), BLACK
            VbMqMtsClientForm.Line (x2 + 1, 0)-(x2 + 1, y), WHITE
            VbMqMtsClientForm.Line (0, y)-(.ScaleWidth, y), BLACK
            VbMqMtsClientForm.Line (0, y + 1)-(.ScaleWidth, y + 1), WHITE
    
            ' now draw the various lines
            ' Flow for Request Transfer ---------------------------
            ' From client screen to MTS object
            '    to BankRequest queue and to database (coordinated)
            
            Dim mtsy1 As Single, mtsy2 As Single, mtsy3 As Single ' convenience
            mtsy1 = 0.3: mtsy3 = 0.85: mtsy2 = (mtsy1 + mtsy3) / 2
            If hi = HRequestTransfer Or hi = HAll Then
                col = COLA
            Else
                col = MUTELINE
            End If
            DrawLine .RequestFrame, 1, 0.7, .MTSFrame, -0, mtsy1, col
            DrawLine .MTSFrame, 0.3, 0, .DBFrame, -999, 1, col
            DrawLine .MTSFrame, 1, mtsy1, .MQFrame, 0, -999, col
            DrawLine .MQFrame, 0.5, 0, .TQOut, -999, 1, col
            DrawLine .TQOut, 1, 0.5, .ShowBankRequest, -0, -999, col
            
            ' Flow for Back End processing ---------------------------
            ' From BankRequest queue to BankReply queue.
            If hi = HBackEnd Or hi = HAll Then
                col = .BackEndFrame.BackColor
            Else
                col = MUTELINE
            End If
            DrawLine .ShowBankRequest, 0.5, 1, .BackEndFrame, -999, 0, col
            DrawLine .BackEndFrame, 0.5, 1, .TQIn, -999, 0, col
            DrawLine .TQIn, 0, 0.5, .ShowBankReply, 1, -999, col
            
            ' Flow for Find Replies (MQ to DB) ---------------------------
            ' From BankReply queue to database (coordinated)
            ' Used directly by the transaction tester, and indirectly by Query Status
            If hi = HFind Or hi = HStatus Or hi = HAll Then
                col = .MQFrame.BackColor
            Else
                col = MUTELINE
            End If
            DrawLine .ShowBankReply, -999, 0, .MQFrame, 0.5, 1, col
            DrawLine .MQFrame, 0, -999, .MTSFrame, 1, mtsy2, col
            DrawLine .MTSFrame, 0.7, 0, .DBFrame, -999, 1, col
            
            ' Last bit of flow for Transaction Tester --------------
            ' From BankReply queue to client screen.
            If hi = HFind Or hi = HAll Then
                col = .MQFrame.BackColor
            Else
                col = MUTELINE
            End If
            DrawLine .MTSFrame, 0, mtsy2, .ReplyFrame, 1, -999, col
            
            ' Last bit of flow for QueryStatus -----------------
            ' From database to client screen.
            If hi = HStatus Or hi = HAll Then
                col = .MTSFrame.BackColor
            Else
                col = MUTELINE
            End If
            DrawLine .MTSFrame, 0, mtsy3, .StatusFrame, 1, 0.3, col
            DrawLine .DBFrame, -999, 1, .MTSFrame, 0.5, 0, col
        End With

        CurrentHighlight = hi
    End If
    
End Sub

' Log a problem.
Public Sub log(msg As String)
    VbMqMtsClientForm.ServerLog.AddItem msg
End Sub

' Empty all messages from queue.
Public Sub ClearQueue(qname As String, QMName As String)
    Dim mqs As New MQSession
    Dim qm As MQQueueManager
    Dim q As MQQueue
    Dim inmsg As MQMessage
    Dim gmo As MQGetMessageOptions
    
    On Error GoTo eee
    
    ' set up MQ objects for input
    Set qm = mqs.AccessQueueManager(QMName)
    Set q = qm.AccessQueue(qname, MQOO_INQUIRE Or MQOO_INPUT_AS_Q_DEF)
    
    ' Once we are here, process all available messages
    ' Set up and get the input message, but only bother with one byte.
    Do
        Set inmsg = mqs.AccessMessage
        Set gmo = mqs.AccessGetMessageOptions
        gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG
                
        On Error Resume Next
        q.Get inmsg, gmo, 1
        On Error GoTo eee
    Loop While q.CompletionCode < MQCC_FAILED
    
    ' Now normal cleanup
    qm.Disconnect
    Set q = Nothing
    Set qm = Nothing
    Exit Sub
    
eee:
    ' handle all errors, but no messages is not an error
    If mqs.ReasonCode = MQRC_NO_MSG_AVAILABLE Then Resume Next
    
    log "Error emptying queue '" & "'" & qname & ": " & Err.Description
    If Not qm Is Nothing Then qm.Disconnect
    Exit Sub
       
End Sub

' Empty the database and the queues, and clear the status boxes.
Public Sub ResetDemo()
    With VbMqMtsClientForm
        DoDBClear .DBShow
        ClearQueue "BankRequest", .QMName.Text
        ClearQueue "BankReply", .QMName.Text
        .ServerLog.Clear
        .NewReplies.Clear
        .StatusResult.Cls
        DrawLines HNone
        RefreshDisplay
    End With
End Sub

' Read tool tips from file and set them onto the graphics objects.
Public Sub ReadToolTips()
    Dim s As String  ' cumulative string for tooltip
    Dim dd As String ' single line
    Dim ff As Long
    Dim cname As String
    
    On Error Resume Next
    ff = FreeFile
    Open "VbMqMts.tt" For Input As ff
    If Err Then Exit Sub  ' Don't complain too much if no tool tips
    While Not EOF(ff)
        Line Input #ff, dd
        If Left(dd, 1) = "[" Then
            If cname <> "" Then VbMqMtsClientForm(cname).ToolTipText = s
            s = ""
            cname = Mid(dd, 2, Len(dd) - 2)  ' assume trailing "]"
        Else
            s = s & dd & " " & Chr(13) & Chr(10) ' formatting such as chr(10) do not seem to work
        End If
    Wend
    Close ff
End Sub

Public Sub MTSSetup()
    
    ' Set up path names
    Dim packageName As String
    packageName = "VbMqMts"
    Dim path As String
    path = CurDir & "\"
    ' allow for running from sample directory as .exe, or from source directory
    If LCase(Right(path, 15)) = "\vbmqmtsclient\" Then path = path & "..\..\"
    
    ' First, we create the catalog object
    Dim catalog As MTSAdmin.catalog
    Set catalog = CreateObject("MTSAdmin.Catalog.1")
    
    ' Then we get the packages collection
    Dim packages As CatalogCollection
    
    Set packages = catalog.GetCollection("Packages")
    packages.Populate
    
    ' Remove all packages that go by the same name as the package we wish to install
    log "Remove old " & packageName & " packages"
    Dim numpackages As Long
    numpackages = packages.count
    Dim i As Long
    For i = numpackages - 1 To 0 Step -1
        If packages.Item(i).Value("Name") = packageName Then
            packages.Remove (i)
        End If
    Next
    
    ' Commit our deletions
    packages.SaveChanges
    
    ' Add a new package
    log "Add new " & packageName & " packages"
    Dim newPackage As CatalogObject
    Set newPackage = packages.Add
    newPackage.Value("Name") = packageName
    newPackage.Value("SecurityEnabled") = "N"
    
    ' Commit new package
    packages.SaveChanges
    
    ' Refresh packages
    packages.Populate
    
    ' Get components collection for new package
    Dim components As CatalogCollection
    Set components = packages.GetCollection("ComponentsInPackage", newPackage.Value("ID"))
    
    ' Install components
    Dim util As ComponentUtil
    Set util = components.GetUtilInterface
    
    ' VB and VC
    log "Add new component to " & packageName
    Dim dllpath As String
    dllpath = path & "VbMqMtsServe.dll"
    util.InstallComponent dllpath, "", ""
    ' dllpath = path & "VcMqMtsServe.dll"
    ' util.InstallComponent dllpath, "", ""
    
    ' Refresh components
    components.Populate
    
    ' Iterate through the components and change the transaction attributes
    Dim numcomponents As Long
    numcomponents = components.count
    
    Dim component As CatalogObject
    Dim progName As String
    
    log "Set up transaction properties on " & packageName
    For i = numcomponents - 1 To 0 Step -1
        Set component = components.Item(i)
        progName = component.Value("ProgID")
            
        ' until stated otherwise
        component.Value("Transaction") = "Required"
    
        If progName = "VbMqMtsServe.QueryStatus" Then
            component.Value("Transaction") = "Not Supported"
        End If
    Next
    
    ' Commit the changes to the components
    components.SaveChanges
End Sub

Public Sub SetupDemo()
    With VbMqMtsClientForm
        .ServerLog.Clear
        If CreateQueue("BankRequest", .QMName.Text) = False Then GoTo failure
        If CreateQueue("BankReply", .QMName.Text) = False Then GoTo failure
        If CreateTable("BankRequest", .DBConnection.Text) = False Then GoTo failure
        MTSSetup
    End With
    
failure:
    
End Sub

' Use database 'sample' if defined
Public Function DefaultDB() As String
    
    Dim Cn As ADODB.Connection
    
    On Error GoTo eee

    Set Cn = New ADODB.Connection
    Cn.Open "Sample"
    Cn.Close
    Set Cn = Nothing
    DefaultDB = "Sample"
    Exit Function
eee:
    Set Cn = Nothing
    DefaultDB = ""
End Function
