VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "RequestTransfer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'<!/******************************************************************************/!>
'<!/*                                                                            */!>
'<!/* Program name: VbMqMts                                                      */!>
'<!/*                                                                            */!>
'<!/* Description:  Sample VB program for WebSphere MQ MTS Support               */!>
'<!/*                                                                            */!>
'<!/* Statement:    Licensed Materials - Property of IBM                         */!>
'<!/*                                                                            */!>
'<!/*               (C) Copyright IBM Corp. 1999 - 2000                          */!>
'<!/*                                                                            */!>
'<!/******************************************************************************/!>
Option Explicit

' This object is designed to run under MTS and to act as
' a simple example server object.
'
' The object receives a single request to post a transfer request
' to a back end via WebSphere MQ.
'
' The assumption is that the back end system is not synchronously available,
' so the object does not wait for a response.
'
' In order to be able to confirm that the transfer request has been made,
' the object should also update a local database table with the request details.
' (not implemented)
'
' WHEN database is implemented:
' In order to ensure that the database table holds correct information
' in case of any failure during execution of the object,
' the WebSphere MQ put and database update are performed under transactional control.
' This object should be registered with MTS as 'requires transaction'.
'


Const TRANCODE_ATR1 = "ATR1"  ' back end transaction code for account transfer
Dim WithEvents Cn As ADODB.Connection
Attribute Cn.VB_VarHelpID = -1
Dim m_Connection As String
Dim m_QmName As String

Public Property Let Connection(s As String)
    m_Connection = s
End Property
Public Property Get Connection() As String
    Connection = m_Connection
End Property
Public Property Let QmName(s As String)
    m_QmName = s
End Property
Public Property Get QmName() As String
    QmName = m_QmName
End Property
Public Property Let uselog(b As Boolean)
    m_log = b
End Property
Public Property Get uselog() As Boolean
    uselog = m_log
End Property

Public Function RequestTransfer(FromAccount As String, ToAccount As String, Amount As Currency) As String
    Dim MessageIdHex As String
    Dim mqr As String, dbr As String
    Dim ok As Boolean
    ok = True ' until proven otherwise
    
    ' perform MQ part of transaction
    If InStr(FromAccount, "NOMQ") = 0 Then
        mqr = RequestTransferMQ(FromAccount, ToAccount, Amount, MessageIdHex)
        If mqr <> "" Then ok = False
    Else
        MessageIdHex = "No MQ call"
        mqr = "No MQ call"
    End If
    
    ' and record the transfer in the database
    ' (no point if mq already failed, but do it for the demo anyway)
    If m_Connection <> "" And InStr(FromAccount, "NODB") = 0 Then
        dbr = RequestTransferDB(MessageIdHex, FromAccount, ToAccount, Amount, "Requested")
        If dbr <> "" Then ok = False
    Else
        dbr = "No DB call"
    End If
    
    ' Could clean up the objects here,
    ' but they will go away as the function ends anyway.
    
    ' Close off the transaction
    If Not GetObjectContext Is Nothing Then
        If ok Then
            GetObjectContext.SetComplete
        Else
            GetObjectContext.SetAbort
        End If
    End If
    
    RequestTransfer = "mq=" & mqr & ": db=" & dbr
End Function

Private Function RequestTransferMQ(ByVal FromAccount As String, ByVal ToAccount As String, ByVal Amount As Currency, MessageIdHex As String) As String
    Dim mqs As MQSession
    Dim qm As MQQueueManager
    Dim q As MQQueue
    Dim msg As MQMessage
    Dim pmo As MQPutMessageOptions
    Dim ctxObject As ObjectContext
   
    On Error GoTo eee
    
    Set ctxObject = GetObjectContext()
    
    ' Check inputs (feeble check, just a demo
    If Len(FromAccount) <> 8 Then
        RequestTransferMQ = "From Account '" & FromAccount & "'wrong length; should be 8."
        Exit Function
    End If
    If Len(ToAccount) <> 8 Then
        RequestTransferMQ = "To Account '" & ToAccount & "'wrong length; should be 8."
        Exit Function
    End If
    If Amount <= 0 Or Amount > 10000 Then
        RequestTransferMQ = "Invalid amount, should be between 0 and 10000."
        Exit Function
    End If
    
    ' prepare the MQ objects
    If ctxObject Is Nothing Then
        Set mqs = CreateObject("MQAX200.MQSession")
    Else
        Set mqs = ctxObject.CreateInstance("MQAX200.MQSession")
    End If
    Set qm = mqs.AccessQueueManager(m_QmName)  ' use the default queue manager
    Set q = qm.AccessQueue("BankRequest", MQOO_OUTPUT Or MQOO_INQUIRE)
    Set msg = mqs.AccessMessage
    Set pmo = mqs.AccessPutMessageOptions
    
    ' set up the contents of the output message
    msg.WriteString TRANCODE_ATR1
    msg.WriteString FromAccount
    msg.WriteString ToAccount
    msg.WriteString Format(Amount, "00000000.00")
    
    ' set up the WebSphere MQ bit to make sure the server responds correctly
    msg.ReplyToQueueName = "BankReply"
    msg.Persistence = MQPER_PERSISTENT
    pmo.Options = MQPMO_SYNCPOINT
    
    ' finally, put the message
    q.Put msg, pmo
    
    ' return the id
    MessageIdHex = msg.MessageIdHex
    
    If GetObjectContext Is Nothing Then
        ' had some code to make it easier to test outside MTS
        RequestTransferMQ = " Warning: request transfer not registered with MTS as requires transaction? MQ one phase commit used."
        qm.Commit ' use MQ standalone transaction completion
    End If
    
    qm.Disconnect
    Set qm = Nothing
    Exit Function
    
eee: ' come here on any error
    RequestTransferMQ = "Error in RequestTransfer" & Err.Number & " " & Hex(Err.Number) & ": " & Err.Description
    Exit Function
End Function

' help for debug, check about transactions
Public Function transactionMode() As String
    If GetObjectContext Is Nothing Then
        transactionMode = "no context"
    ElseIf GetObjectContext.IsInTransaction Then
        transactionMode = "in transaction"
    Else
        transactionMode = "not in transaction"
    End If
End Function

' If a connection name is given, the request information is recorded in the database.
' If the database does not hold a BankRequest table, one is created.
' This creation has only been tested against Jet database engine.
' Field types are probably wrong for real SQL databases.
'
' The database update will be coordinated with the MQ put with an appropriate database.
Private Function RequestTransferDB(Id As String, FromAccount As String, ToAccount As String, Amount As Currency, Status As String) As String

    Dim rs As New ADODB.Recordset
    Dim rr As String
    
    If m_Connection = "" Then
        log "no database conneciton made, no connect string"
        RequestTransferDB = "no db specified"
        Exit Function
    Else
        log "connection string: " & m_Connection
    End If
    On Error Resume Next

    Set Cn = New ADODB.Connection
    
    'DSN-Less connection for SQL Server
    ' Cn.Open "Driver={SQL Server};Server=ONION;Uid=sa;Pwd=;Database=master"
    log "about to cn.open"
    Cn.Open m_Connection
    rr = rr & checkerr

    'log "about to begin transaction"
    'Cn.BeginTrans
    'rr = rr & checkerr
    
    ' Open titles table.
    log "about to rs.open"
    rs.Open "SELECT ID, FromAccount, ToAccount, Amount, Status, ChangeTime FROM BankRequest", Cn, adOpenDynamic, adLockPessimistic
'    If Err Then
'        rr = rr & checkerr
'        log "open failed, try to create table"
'        rr = "creating new table. "
'        Cn.Execute "create table BankRequest (ID char(48), FromAccount char(8), ToAccount char(8), Amount currency, Status char(20), ChangeTime char(20) )"
'        rr = rr & checkerr
'        rs.Open "SELECT ID, FromAccount, ToAccount, Amount, Status, ChangeTime FROM BankRequest", Cn, adOpenDynamic, adLockPessimistic
'    End If
    rr = rr & checkerr
    
    'Create an editable record
    rs.AddNew
    rr = rr & checkerr
    rs!Id = Id
    rr = rr & checkerr
    rs!FromAccount = FromAccount
    rr = rr & checkerr
    rs!ToAccount = ToAccount
    rr = rr & checkerr
    rs!Amount = Amount
    rs!Status = Status
    rr = rr & checkerr
    rs!ChangeTime = Now
    rr = rr & checkerr
    
    rs.Update
    rr = rr & checkerr
    rs.Close
    rr = rr & checkerr
   
    Cn.Close
    rr = rr & checkerr
    
    RequestTransferDB = rr
    Set rs = Nothing
    Set Cn = Nothing
    Exit Function
    
eee:
    Dim e As String
    e = "Error " & Err.Number & " " & Err.Description
    log e
    RequestTransferDB = e
    Set rs = Nothing
    Set Cn = Nothing
    Exit Function
End Function

Private Sub Class_Initialize()
    m_Connection = ""
    log "class initialize" ' this will never actaully happen, as m_log will be false
End Sub

Private Sub Class_Terminate()
    log "class terminate"
End Sub

Private Sub Cn_CommitTransComplete(ByVal pError As ADODB.Error, adStatus As ADODB.EventStatusEnum, ByVal pConnection As ADODB.Connection)
    log "cn: commit done"
End Sub

Private Sub Cn_ConnectComplete(ByVal pError As ADODB.Error, adStatus As ADODB.EventStatusEnum, ByVal pConnection As ADODB.Connection)
    log "cn: connect done"
End Sub

Private Sub Cn_Disconnect(adStatus As ADODB.EventStatusEnum, ByVal pConnection As ADODB.Connection)
    log "cn: disconect done"
End Sub

